#include "T6713Driver.h"

T6713::T6713() {
    _i2caddr = T6713_I2CADDR_DEFAULT;
}

bool T6713::begin() {
    bool success = true;

    if( this->readStatus() != 0) {
        Serial.print("Error when reading status at startup: ");
        Serial.println(this->readCO2value());
        success = false;
    }
    else {
        /*
        Serial.print("Able to read status: ");
        Serial.println(this->_status, HEX);
        */
    }
    return success;
}

/** Reads the CO2 value from T6713
 * 
 * The result is read into local variable _CO2value
 * 
 * @return Output from twi_writeTo function
 *          0 .. success
 *          1 .. length to long for buffer
 *          2 .. address send, NACK received
 *          3 .. data send, NACK received
 *          4 .. other twi error (lost bus arbitration, bus error, ..)
 */
uint8_t T6713::readCO2value() {
    // Set up address to communicate with
    Wire.beginTransmission(_i2caddr);
    // Write commands to read from CO2 register, address:0x138B
    Wire.write(0x04); // Read input registers
    Wire.write(0x13); // MSB of register address
    Wire.write(0x8B); // LSB of register address
    Wire.write(0x00); // MSB of number of input registers to read
    Wire.write(0x01); // LSB of number of input registers to read

    // Complete transmission. This carries out the transmission
    uint8_t ret = Wire.endTransmission();

    if(ret != 0) {
        return ret;
    }
    //-------2nd, read out result from sensor------------
    delay(10); // Wait for result to be ready
    Wire.requestFrom(_i2caddr, 4); // The response will be 4 bytes long
    _fCode = Wire.read();
    _byteCount = Wire.read();
    // Read MSB and LSB into private 16-bit register 
    _CO2value = (Wire.read()<<8); // TODO in co2meter application note this is &ed with 0x3F before left shift. Why?
    _CO2value |= Wire.read();
    return ret;
}

uint16_t T6713::getCO2value() {
    return _CO2value;
}

uint8_t T6713::reset() {
    Wire.beginTransmission(_i2caddr);

    Wire.write(0x05); // Write single coil
    Wire.write(0x03); // Address MSB
    Wire.write(0xE8); // Address LSB
    Wire.write(0xFF); // Output value MSB
    Wire.write(0x00); // Output value LSB

    return Wire.endTransmission();
}

uint16_t T6713::getStatus() {
    return _status;
}

uint8_t T6713::readStatus() {
    Wire.beginTransmission(_i2caddr);

    Wire.write(0x04); // Read input register
    Wire.write(0x13); // MSB register address
    Wire.write(0x8A); // LSB register address
    Wire.write(0x00); // Input registers to read MSB
    Wire.write(0x01); // Input registers to read LSB.

    // Complete transmission
    uint8_t ret = Wire.endTransmission();

    if(ret != 0) {
        return ret;
    }

    // store response in _status variable
    delay(10); // wait for result to be ready
    _fCode = Wire.read();
    _byteCount = Wire.read();
    _status = (Wire.read()<<8);
    _status |= Wire.read();

    return ret;
}

bool T6713::isInWarmupMode() {
    this->readStatus();
    return _status & 0x0800;

}

uint16_t T6713::getAddress() {
    Wire.beginTransmission(_i2caddr);

    Wire.write(0x03); // Read holding register
    Wire.write(0x0F); // Register address MSB
    Wire.write(0xA5); // Register address LSB
    Wire.write(0x00); // Registers to read MSB
    Wire.write(0x01); // Registers to read LSB

    //Complete transmission
    Wire.endTransmission();

    delay(10); // Wait for result to be ready
    uint16_t address;
    _fCode = Wire.read();
    _byteCount = Wire.read();
    address = (Wire.read()<<8);
    address |= Wire.read();

    return address;
}

bool T6713::isMatchingAddress() {
    return (_i2caddr == this->getAddress());
}
