#include <Arduino.h>
#include <Adafruit_BME280.h>
#include <Adafruit_SGP30.h>
#include <avr/sleep.h>
#include "Timer.h"
#include <EEPROM.h>

Adafruit_BME280 myBME280;
Adafruit_SGP30 mySGP30;

#define SIZE_RX_COMMAND 5
char rx_command[SIZE_RX_COMMAND];
#define LED_PIN 2

#define BASELINE_ECO2_LOW 0
#define BASELINE_ECO2_HIGH 1
#define BASELINE_TVOC_LOW 2
#define BASELINE_TVOC_HIGH 3

float temperature;
float humidity;
float pressure;

int counter = 0;
int seconds = 0;

bool read_BME280 = false;
bool read_SGP30 = false;
bool LED_on = false;
bool read_serial = false;
bool check_serial = false;
bool hour_passed = false;

void setup() {
    cli();

    // Setup of LED
    pinMode(LED_PIN, OUTPUT);
    digitalWrite(LED_PIN, LOW);
    
    Serial.begin(9600);
    Serial.println("-");

    set_sleep_mode(SLEEP_MODE_IDLE);
    sei();
    
    if(! myBME280.begin(0x76)) {
        Serial.println("Err");
    } 
    
    if(! mySGP30.begin()) {
        Serial.println("Err");
    }
    else {
        /*
        uint16_t baseline_eco2;
        uint16_t baseline_tvoc;
        read_eeprom_baselines(&baseline_eco2, &baseline_tvoc);
        mySGP30.setIAQBaseline(baseline_eco2, baseline_tvoc);
        */
    }

    Timer_init();
    
}

void loop() {

    if(read_SGP30) {
        mySGP30.IAQmeasure();
        read_SGP30 = false;
    }
    if(read_BME280) {
        temperature = myBME280.readTemperature();
        humidity = myBME280.readHumidity();
        pressure = myBME280.readPressure();
        read_BME280 = false;
    }
    if(hour_passed) {
        hour_passed = false;
        store_baselines();
    }
    if(check_serial) {
        if(Serial.available() > 0) {
            Serial.readBytes(rx_command,SIZE_RX_COMMAND);
            if(!strcmp(rx_command, "get\n")) {
                print_sensors();
            }
        }
        check_serial = false;
    }


    sleep_mode();
}

ISR(TIMER2_COMPA_vect) {
    // One second elapsed
    if(counter == 125) {
        counter = 0;
        seconds ++;
        read_SGP30 = true;

        // every 10 seconds
        // Read BME280, heartbeat LED blink
        if(seconds % 10 == 0) {
            read_BME280 = true;
            digitalWrite(LED_PIN, HIGH);
            LED_on = true;
        }
        // one hour passed. Restart seconds counter.
        // stores baseline values
        if(seconds == 3600) {
            seconds = 0;
            hour_passed = true;
        }

    }

    // Turn of LED for short blink
    else if(counter == 5 && LED_on) {
        digitalWrite(LED_PIN, LOW);
        LED_on = false;
    }

    // Check for new UART message
    if(counter % 2 == 0) {
        check_serial = true;
    }
    counter ++;
}

void print_sensors() {
    Serial.print(temperature);
    Serial.print(",");
    Serial.print(humidity);
    Serial.print(",");
    Serial.print(pressure);
    Serial.print(",");
    Serial.print(mySGP30.eCO2);
    Serial.print(",");
    Serial.println(mySGP30.TVOC);
}

void store_baselines() {
    uint16_t baseline_eco2;
    uint16_t baseline_tvoc;

    mySGP30.getIAQBaseline(&baseline_eco2, &baseline_tvoc);
    
    // TODO remove
    Serial.print("Baselines: ");
    Serial.print(baseline_eco2, HEX);
    Serial.print(",");
    Serial.println(baseline_tvoc, HEX);

    EEPROM.write(BASELINE_ECO2_LOW, (uint8_t) (baseline_eco2 & 0xFF));
    EEPROM.write(BASELINE_ECO2_HIGH, (uint8_t) ((baseline_eco2>>8) & 0xFF));

    EEPROM.write(BASELINE_TVOC_LOW, (uint8_t) (baseline_tvoc & 0xFF));
    EEPROM.write(BASELINE_TVOC_HIGH, (uint8_t) ((baseline_eco2>>8) & 0xFF));
}

void read_eeprom_baselines(uint16_t * eco2, uint16_t * tvoc) {
    *eco2 =  (EEPROM.read(BASELINE_ECO2_HIGH)<<8) | (EEPROM.read(BASELINE_ECO2_LOW) & 0xFF);
    *tvoc += (EEPROM.read(BASELINE_TVOC_HIGH)<<8) | (EEPROM.read(BASELINE_TVOC_LOW) & 0xFF);
}